function [F0Feature, F0Location, F0Track] = CalculateF0Features_HarmonicStructure(PeakData, PeakAmpData, EstF0, para)
% Calculate F0 harmonic structure feature vectors
% Input
%   - PeakData      : Frequency (MIDI number) of extracted peaks of the audio file, each column is a frame
%   - PeakAmpData   : Amplitudes (dB) of extracted peaks of the audio file, each column is a frame
%   - EstF0         : Multiple F0 estimation results (MIDI number), each column is a frame
%   - para
%       - maxHarm   : The number of harmonics, i.e. harmonic structure feature dimensionality
%       - Thresh_PeakF0Belong   : The threshold (in MIDI number) in frequency deviation to decide which peak belongs to which F0
% Output
%   - F0Feature     : Harmonic structure of this F0. Each row is a feature vector. Total energy of harmonics of each F0 is normalized to 100 dB.
%   - F0Locations
%       - Col 1     : Frame number
%       - Col 2     : Frequency (MIDI number)
%       - Col 3     : Energy of its harmonics (20 * log10 (sum linear amplitudes))
%   - F0Track       : F0 track label
%
% Author: Zhiyao Duan
% Created: 4/28/2009
% Last modified: 7/2/2012

fprintf('Calculating harmonic structure vectors...');

% parameters
Thresh_PeakF0Belong = para.Thresh_PeakF0Belong;
maxHarm = para.maxHarm;

[MaxPoly, FrameNum] = size(EstF0);

% Initialize several variables
F0Features = zeros(MaxPoly*FrameNum, maxHarm);                      % feature vectors
F0Locations = zeros(MaxPoly*FrameNum, 3);
F0Tracks = zeros(MaxPoly*FrameNum, 1);                              % Initial F0 tracks

% Start processing each frame
feaNum = 0;                                                         % count the number of feature vectors
for i = 1:FrameNum
    if mod(i, 10) == 0
        fprintf('.');
    end
    if mod(i, 1000) == 0
        fprintf('\n');
    end
    
    if sum(PeakData(:,i)~=0) == 0                                   % this frame is silent or there is no peak in the spectrum
        continue;
    end
    
    peak = PeakData(:, i);
    peakAmp = PeakAmpData(:, i);
    
    idx = peak~=0;                                                  % because the peak number in each frame might be less than 100, so there are some 0s
    peak = peak(idx);
    peakAmp = peakAmp(idx);    
       
    mpeak = peak;                                                   % peak frequency in MIDI number
    fpeak = midi2hz(mpeak);                                         % peak frequency in Hz
    
    for j = 1:MaxPoly
        if EstF0(j, i) == 0                                        
            continue;
        end        
        
        mF0 = EstF0(j, i);                                          % F0 in MIDI number
        fF0 = midi2hz(mF0);                                         % F0 in Hz
    
        Ratio = fpeak / fF0;                                        % calculate the harmonic number of peak i given F01
        RRatio = round(Ratio);
      
        % Use frequency deviation to decide which peak can belong to which F0
        err = mpeak - mF0 - 12*log2(max(RRatio,1));                 % the frequency deviation of all peaks from harmonics of F01
        
        % Calculate the linear harmonic structure
        F0HarmStruct = zeros(1, maxHarm);
        for k = 1:maxHarm
            harmidx = find((abs(err)<Thresh_PeakF0Belong & (RRatio == k)) == 1);   % find all peaks that could belong to this harmonic
            if isempty(harmidx)
                F0HarmStruct(k) = 0;                      % if no peak is associated, use 0. This is much better!
            else                
                [~, idx] = min(abs(err(harmidx)));
                F0HarmStruct(k) = 10^(peakAmp(harmidx(idx))/20);   	% if there are multiple peaks, use the closest one
            end
        end
        
        % normalize harmonic structure to have the total energy 100 dB for each F0's harmonics
        idx = F0HarmStruct ~= 0;
        tmpLinearHarmonic = F0HarmStruct(idx);
        F0Energy = 20*log10(sum(tmpLinearHarmonic));          % total energy of the harmonics of this F0 (dB)
        F0HarmStruct(idx) = 20*log10(tmpLinearHarmonic / sum(tmpLinearHarmonic) * 1e5);
        
        % compose the feature vector for each F0
        feaNum = feaNum + 1;
        F0Features(feaNum, :) = F0HarmStruct;
        F0Locations(feaNum, 1) = i;
        F0Locations(feaNum, 2) = EstF0(j, i);
        F0Locations(feaNum, 3) = F0Energy;
        F0Tracks(feaNum) = j;                                       % An initial track label for each F0
    end
end

% We collected feaNum features in total
F0Feature = F0Features(1:feaNum, :);
F0Location = F0Locations(1:feaNum, :);
F0Track = F0Tracks(1:feaNum);

fprintf('\n');