function CannotLink = FindAllCannotLinks(F0Location, Group, F0Group, para)
% Find all the cannot-links in the multi-pitch tracking problem, it is
% based on the must-links. A cannot-link is imposed on a pair of concurrent pitches,
% if the notelets (groups) of which the pitches is contained "overlap".
% There is a threshold to determine if two notelets overlap or not.
% Input
%   - F0Location        : Location of each F0, each row corresponds to a F0
%   - Group             : F0 groups of must-links, each row is group
%   - F0Group           : the mapping from F0 to Group, indicating which group the F0 is in
%   - para              : parameters
% Output
%   - CannotLink        : the cannot-link lists for each pitch
% 
% Author: Zhiyao Duan
% Created: 5/28/2009
% Last modified: 6/15/2012

fprintf('Finding all cannot-links...');

CNL_opth = para.CNL_opth;               % the overlap threshold (in frames) to decide if two notelet overlap
feaNum = size(F0Location, 1);           % number of pitches

CannotLink = cell(feaNum, 1);

for i = 1:feaNum
    if mod(i, 10) == 0
        fprintf('.');
    end
    if mod(i, 1000) == 0
        fprintf('\n');
    end
    
    CannotLink{i} = zeros(1, feaNum);   % the cannot-link list for each pitch
    CannotLinkNum = 0;

    Atime = F0Location(i, 1);           % the time of pitch A
    
    for j = 1:feaNum
        Btime = F0Location(j, 1);       % the time of pitch B
        if i~=j && Btime==Atime
            if CNL_opth == 0            % if the CNL_opth is 0, then a cannot link is always imposed between any pair of concurrent pitches
                CannotLinkNum = CannotLinkNum + 1;
                CannotLink{i}(CannotLinkNum) = j;
            else                        % otherwise, decide the cannotlink according to the threshold
                % the onset and offset of the notelet (group) that contains A
                Agroup = F0Group(i);
                Aonset = F0Location(Group{Agroup}(1), 1);
                Aoffset = F0Location(Group{Agroup}(end), 1);
                % the onset and offset of the notelet (group) that contains B
                Bgroup = F0Group(j);
                Bonset = F0Location(Group{Bgroup}(1), 1);
                Boffset = F0Location(Group{Bgroup}(end), 1);
                % the two notelets overlap long enough
                if Aonset+CNL_opth <= Boffset && Bonset+CNL_opth <= Aoffset     
                    CannotLinkNum = CannotLinkNum + 1;
                    CannotLink{i}(CannotLinkNum) = j;
                end
            end
        end
    end    
    CannotLink{i} = CannotLink{i}(1:CannotLinkNum);
end
fprintf('\n');