function [F0Track, CurrMustLink, CurrCannotLink] = IncrementalConstrainedClustering_ConstraintsOnly(F0Feature, F0Track, AllMustLink, AllCannotLink, para)
% Perform incremental constrained clustering from an initial clustering.
% Ignore the objective function, only pay attention to increasing the
% number of satisfied constraints.
%
% Input:
%   - F0Feature     : F0 features, each row corresponds to a F0
%   - F0Track       : initial track labels of F0s
%   - AllMustLink   : all must-link constraints
%   - AllCannotLink : all cannot-link constraints
%   - para
%       - DistType      : 'Euclidean' (default) or 'Mahalanobis' distance
% Output:
%   - F0Track       : learned final track labels of F0s
%   - CurrMustLink  : currently satisfied must-links by the final clustering
%   - CurrCannotLink: currently satisfied cannot-links by the final clustering
% 
% Author: Zhiyao Duan
% Created: 6/15/2012
% Last modified: 6/15/2012

% Memory allocation
Total_J = zeros(1000, 1);                                           % Record values of the objective function at each step
MustLink_Num = zeros(1000, 1);                                      % Record the number of must-links that are satisfied at each step
CannotLink_Num = zeros(1000, 1);                                    % Record the number of cannot-links that are satisfied at each step

% Find all satisfied must-link constraints by the initial clustering
CurrMustLink = FindCurrMustLinks(AllMustLink, F0Track);         
% Find all satisfied cannot-links constraints by the initial clustering. In
% this initialization, every cannot-link is satisfied.
CurrCannotLink = AllCannotLink; 

% Start iterations
step = 1;
Total_J(step) = CalculateIntraClassDist(F0Feature, F0Track, para.DistType);
MustLink_Num(step) = CalculateConstraintNum(CurrMustLink);
CannotLink_Num(step) = CalculateConstraintNum(CurrCannotLink);
AllMustLink_Num = CalculateConstraintNum(AllMustLink);
AllCannotLink_Num = CalculateConstraintNum(AllCannotLink);
fprintf('Step: %d \t IntraClassDist: %f; MustLinkNum: %d/%d; CannotLinkNum:%d/%d.\n', step, Total_J(step), MustLink_Num(step), AllMustLink_Num, CannotLink_Num(step), AllCannotLink_Num);

while 1
    step = step + 1;
    
    % Find a new clustering that has smaller intra-class distance and
    % satisfies more must-links. Update the satisfied must-link constraints
    [F0Track, Total_J(step), CurrMustLink, flag_new] = FindNewClustering_ConstraintsOnly(F0Feature, F0Track, CurrMustLink, CurrCannotLink, AllMustLink, para);

    % Calculate and output some values of the current clustering
    MustLink_Num(step) = CalculateConstraintNum(CurrMustLink);
    CannotLink_Num(step) = CalculateConstraintNum(CurrCannotLink);
    if mod(step, 10)==1
        fprintf('Step: %d \t IntraClassDist: %f; MustLinkNum: %d/%d; CannotLinkNum:%d/%d.\n', step, Total_J(step), MustLink_Num(step), AllMustLink_Num, CannotLink_Num(step), AllCannotLink_Num);
    end
    
    % Cannot find a new and better clustering anymore, terminates
    if flag_new == 0                                                
        break;
    end    
end